/**
 * @file Blitz_click_PIC32.c
 * @brief <h2> Blitz Click Board Example </h2>
 *
 * @par
 * Example for <a href="http://www.mikroe.com">MikroElektronika's</a>
 * Blitz click board.
 *
 *
 *
 ******************************************************************************/

/**
 * @page Example
 * @date        08/11/2016
 * @author      Branislav Marton
 * @copyright   GNU Public License
 * @version     1.0.0 - Initial testing and verification
 */

/**
 * @page TEST_CFG Test Configurations
 *
 * ### Test configuration STM : ###
 * @par
 * -<b> MCU           </b> :    PIC32MX795F512L
 * -<b> Dev. Board    </b> :    EasyPIC Fusion v7
 * -<b> Oscillator    </b> :
 * -<b> Ext. Modules  </b> :    BLitz Click Board
 * -<b> SW            </b> :    MikroC PRO for PIC32 v3.6.0
 *
 */

/**
 * @page Dev. board setup
 *
 * 1) Put BLITZ click in mikroBUS 1 slot.
 * 2) Button press level for PORTA&C set to GND.
 * 3) Set pull-up for RA4 and RA1.
 */

/**************************************************************************
* Includes
**************************************************************************/
#include <stdint.h>
#include "tft_resources.h"

/**************************************************************************
* Module Preprocessor Constants
**************************************************************************/
#define POLLING_PERIOD 10

#define MODE_TEXT_Y 75
#define TORCH_TEXT_X 20
#define FLASH_TEXT_X 100
#define READY_TEXT_X 180


/**************************************************************************
* Module Typedefs
**************************************************************************/
typedef enum state 
{
    STATE_READY,
    STATE_CHARGE,
    STATE_TORCH,
    STATE_FLASH
} State;

/**************************************************************************
* Module Variable Definitions
**************************************************************************/
sbit FLASH at RC1_bit;
sbit TORCH at RD0_bit;
sbit CHRG at RB8_bit;
sbit FLAG at RE8_bit;

uint8_t buttonstate_center = 0xFF;
uint8_t buttonstate_up = 0xFF;

State state;
State next_state;

/******************************************************************************
* Function Prototypes
*******************************************************************************/
void MCU_Init();
uint8_t Button_Pressed(unsigned int *port, unsigned int pin, uint8_t* state);
void Write_Text(unsigned char* text, unsigned x, unsigned y, unsigned int color);

void Write_Mode_Text(
    uint8_t write_torch,
    unsigned int torch_color, 
    uint8_t write_flash,
    unsigned int flash_color,
    uint8_t write_ready,
    unsigned int ready_colort
);

/******************************************************************************
* Public Functions
*******************************************************************************/
void main() 
{
    MCU_Init();

    state = STATE_CHARGE;
    next_state = STATE_CHARGE;

    while(1) 
    {
        switch(state) 
        {
            case STATE_CHARGE:
                Write_Mode_Text(1, CL_GRAY, 1, CL_BLACK, 1, CL_GRAY);
                FLASH = 0;
                TORCH = 0;
                CHRG  = 1;

                if(!FLAG) 
                    next_state = STATE_READY;

                if(Button_Pressed(&PORTA, 4, &buttonstate_up))
                    next_state = STATE_TORCH;    

                break;

            case STATE_READY:
                Write_Mode_Text(0, 0, 0, 0, 1, CL_BLACK);
                if(Button_Pressed(&PORTA, 1, &buttonstate_center))
                    next_state = STATE_FLASH;

                if(Button_Pressed(&PORTA, 4, &buttonstate_up))
                    next_state = STATE_TORCH;
                    
                break;

            case STATE_FLASH:
                Write_Mode_Text(0, 0, 0, 0, 1, CL_GRAY);
                CHRG = 0;

                FLASH = 1;
                Delay_ms(300);
                FLASH = 0;

                next_state = STATE_CHARGE;
                break;

            case STATE_TORCH:
                Write_Mode_Text(1, CL_BLACK, 1, CL_GRAY, 1, CL_GRAY);
                TORCH = 1;
                FLASH = 0;
                CHRG  = 0;

                if(Button_Pressed(&PORTA, 4, &buttonstate_up))
                    next_state = STATE_CHARGE;
                
                break;
        }
        
        state = next_state;
        Delay_ms(POLLING_PERIOD);
    }
}

void MCU_Init() 
{
    AD1PCFG = 0xFFFF;  // Turning all analog pins to digital
    
    // TORCH pin output.
    TRISD0_bit = 0;
    // CHRG pin output.
    TRISB8_bit = 0;
    // FLASH pin output.
    TRISC1_bit = 0;
    
    // FLAG pin input.
    TRISE8_bit = 1;
    // Joystic CENTER button.
    TRISA4_bit = 1;
    // Joystic UP button.
    TRISA1_bit = 1;

    FLASH = 0;
    TORCH = 0;
    CHRG  = 0;

    display_init();
    
    Write_Mode_Text(1, CL_GRAY, 1, CL_GRAY, 1, CL_GRAY);

    TFT_Set_Font( &Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL );
    TFT_Write_Text( "Note: Use joystick UP button to change mode.", 19, 150 );
    TFT_Write_Text( "Use joystick CENTER button to FLASH when ready.", 19, 167 );
    
    TFT_Set_Font( &Verdana12x13_Regular, CL_RED, FO_HORIZONTAL );
    TFT_Write_Text( "CAUTION: Do not look directly at LED's.", 19, 200 );

    Delay_ms(100);
}

uint8_t Button_Pressed(unsigned int *port, unsigned int pin, uint8_t* state) 
{
    uint8_t new_state = Button(port, pin, 1, 1);
    if(*state == 0xFF && new_state == 0x00 ) 
    {
        *state = new_state;
        return 0xFF;
    }

    *state = new_state;
    return 0x00;
}

void Write_Text(unsigned char* text, unsigned x, unsigned y, unsigned int color)
{
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, color, FO_HORIZONTAL);
    TFT_Write_Text(text, x, y);
}

void Write_Mode_Text(
    uint8_t write_torch,
    unsigned int torch_color, 
    uint8_t write_flash,
    unsigned int flash_color,
    uint8_t write_ready,
    unsigned int ready_color
)
{
    if(write_torch) Write_Text("TORCH", TORCH_TEXT_X, MODE_TEXT_Y, torch_color);
    if(write_flash) Write_Text("FLASH", FLASH_TEXT_X, MODE_TEXT_Y, flash_color);
    if(write_ready) Write_Text("READY", READY_TEXT_X, MODE_TEXT_Y, ready_color);
}
/*************** END OF FUNCTIONS *********************************************/